function resultNO3=CalculateNO3(absorption, sampleSalinity, sampleTemperature, DateTime, ...
                                offsetCorrection, showPlot)
%% Computation of Nitrate concentrations based on absorption spectra from OPUS / ProPS (TriOS) 
%  Before using this function import your data with respective file for OPUS or ProPS 
%  Read the "How to Calculate NO3" pdf for a short description of the processing and needed data formats.

%  Things you have to to determine in the laboratory: 
%  - Collect a reference salinity spectrum (ReferenceSalinity) with your instrument using low nutrient seawater (35 psu).
%  - Adapt the temperature of your salinity reference (calibrateTemperature).  
%  - Collect a Nitrate reference spectra (ReferenceNitrate) with your instrument.
%  - Adapt the Nitrate concentration before calculating NO3.
%  - Check the wavelenght range of your instrument. 

%  Further needed functions are attached at the end of this function (as not built-in MATLAB functions). 
%  Also included as extra functions (see attached folder). 
% 
% Function input:
%  absorption*: 		Absorption spectra in a matrix from OPUS / ProPS
%  sampleSalinity*: 	Table of measured salinities [] from additonal used sensors 
%  sampleTemperature*: 	Table of measured temperature [C] from additonal used sensors
%  DateTime*:           Correpsonding date and time of measurements 
%  offsetCorrection: 	Method of offset correction has to be choosen 
% 	 					'Last': 		Last value will be subtracted from the spectrum
%  						'Linear': 		Offset is deteremined using a linear interpolation
% 	 					'Quadratic': 	Offset is determined using a quadratic interpolation 
%  showPlot: 			'0': Plots won't be shown;'1': Plots will be shown 
%  *must be loaded manually to workspace before using the function.
%
% Function ouput:
%  resultNO3: 			NO3 concentrations in [Mol/L] for each observation.
%
% Contact/Email: 		daniela.meier@uni-oldenburg.de
% 						daniela.voss@uni-oldenburg.de
% 						oliver.zielinski@uni-oldenburg.de
%
% References:
%
% Frank, C; Meier, D; Vo, D; Zielinski, O (2014)  
%  Computation of nitrate concentrations in coastal waters using an in situ 
%  ultraviolet spectrophotometer: Behavior of different computation methods 
%  in a case study a steep salinity gradient in the southern North Sea.
%  Methods in Oceanography. Volume 9, April 2014, Pages 34-43.
%  doi: 10.1016/j.mio.2014.09.002

% Sakamoto, C M; Johnson, K S; Coletti, L J (2009). 
%  Improved algorithm for the computation of nitrate concentrations in 
%  seawater using an in situ ultraviolet spectrophotometer. 
%  Limnol.Oceanogr.: Methods 7, 132-143. Doi: 10.4319/lom.2009.7.132

% Zielinski, O; Vo, D; Saworski, B; Fiedler, B; Koertzinger, A (2011) 
%  Computation of nitrate concentrations in turbid coastal waters using 
%  an in situ ultraviolet spectrophotometer. J Sea Res 65: 456-460.
%  doi: 10.1016/j.seares.2011.04.002 


%% Load your input variables 
% load lab reference spectra for salinity and nitrate determined with your instrument in the laboratory
load ReferenceSalinity.mat 		% absorption spectrum of lab reference (salinity)
load ReferenceNitrate.mat 		% absorption spectrum of lab reference (nitrate)

% load specific wavelengths of your device
load wavelength.mat 	% table of specific wavelengths of used device

% load absorption spectra as matricesmeasured with your instrument (see Import.m for respective instrument) 
load absorption.mat 	% measured absorption spectra from OPUS/ProPS


%% Absorption spectra will be croped to the wavelength range of 200-260 nm
index = find(wavelength<=260 & wavelength>=200);
wavelength_raw = wavelength;
absorption_raw = absorption;
wavelength = wavelength(index);
absorption = absorption(index);
ReferenceNitrate = ReferenceNitrate(index);

%% A temperature and salinity correction of spectra based on data of additonal sensors, e.g. CTD data, will be done
% Therefore, additional functions needed: ESW, ASW and ASE (Sakamoto et al., 2009), no changes are needed.
% All these functions are attached at the end of "calculateNO3" as not built-in functions.
% A reference spectra for salinity (Salref.mat) was measured with the instrument in the lab (low nutrient seawater, 35 psu).  
% Temperature of salinity reference (calibrateTemperature). In our case: 20.56 C. You have to adap the temperature to your conditions. 

calibrateTemperature = 20.56; 		% ADAPT: YOU HAVE TO ADAPT THE TEMPERATURE TO YOUR LAB CONDITIONS 
ESWCalibrated = ReferenceSalinity(index); 		% Salinity lab reference spectrum of low nutrient seawater (35 psu)
ESWData = ESW(wavelength, sampleTemperature, calibrateTemperature ,ESWCalibrated); 
ASEData = ASE(ESWData, sampleSalinity);     
absorptionTSCorrected = absorption - ASEData;


%% A CDOM offset correction will be performed in the wavelength range of 240-260 nm
% This CDOM offset correction has to be done to compensate CDOM absorption signals. 
% An adaption of wavelenght range is needed when using a new device.
% You have to choose in the function the kind of offset correction needed for your observations. 
% Use 'Last' within high turbid waters and low CDOM concentrations.
% Use 'Linear' at intermediate CDOM concentrations.
% Use 'Quadratic' at high CDOM concentrations.

switch offsetCorrection
    case 'Last' % Turbid waters with low CDOM concentrations 
        absorptionOffsetCorrected = absorptionTSCorrected - ...
                                    absorptionTSCorrected(end);
    case 'Linear' % Intermediate CDOM concentrations. 
         fit = polyfit(wavelength(50:74), absorptionTSCorrected(50:74), 1); % ADAPT TO YOUR WAVELENGHT RANGE IF NEEDED
         steigungsVektor = fit(1);
         offsetVector = fit(2);
         absorptionOffsetCorrected = absorptionTSCorrected - ...
                                    (wavelength * steigungsVektor + ...
                                     offsetVector);
    
    case 'Quadratic' % For high CDOM concentrations. 
        fitq = polyfit(wavelength(50:74),absorptionTSCorrected(50:74),2); % ADAPT TO YOUR WAVELENGHT RANGE IF NEEDED 
        pvq = polyval(fitq,wavelength);
        absorptionOffsetCorrected = absorptionTSCorrected - pvq;
            
    otherwise 
        warndlg(' Statement for offset correction must be "Last" or "Linear"  or "Quadratic"..');         
end
assignin('base', 'Restspectra', absorptionOffsetCorrected)
assignin('base', 'wavelength', wavelength);


%% Nitrate computation will be done in the wavelength range of 217-240 nm using linear regression
% An adaption of the wavelenght range is needed when using a new device.
 
fit1 = polyfit(ReferenceNitrate(22:49), absorptionOffsetCorrected(22:49), 1);  %ADAPT TO YOUR WAVELENGHT RANGE IF NEEDED

% Results for nitrate concenrations will be in [mol/L]
% You have to adapt the Nitrate concentration of the Nitrate reference spectra.
% In this case: reference spectra of Nitrate has a concentration of 22.5 mol/L. 

resultNO3 = 22.5 * fit1(1); %ADAPT NITRATE REFERENCE CONCENTRATION


%% Export results to ASCII-file
% Create cell array containing date and time, calculated Nitrate
% concentrations
Data=[DateTime num2cell(resultNO3)];

% Write data to a new file 'resultNitrate.dat'
SaveFileName='resultNitrate.dat';
DataFormat=['%s',repmat('\t%.2f',1,size(resultNO3,2)),'\r\n'];
% HeaderFormat=['%s\t%s\r\n'];
% Header=[{'DateTime'} {'Nitrate [mol/L]'}];

fid=fopen(SaveFileName,'a+');
% fprintf(fid,HeaderFormat,Header{1,:});
for z=1:size(Data,1)
     fprintf(fid,DataFormat,Data{z,:});
end
fclose(fid);


%% Plot results
if showPlot == 1
   subplot(2,2,1), plot(wavelength_raw, absorption_raw); %raw spectra
   hold on
   subplot(2,2,2), plot(wavelength, absorption); %cropped spectra
   hold on
   subplot(2,2,3), plot(wavelength, absorptionTSCorrected); %T-S corrected spectra
   hold on
   subplot(2,2,3), plot(wavelength, wavelength * steigungsVektor + ...
                        offsetVector, 'r'); %Used offset to subtract from spectra
   subplot(2,2,4), plot(wavelength, absorptionOffsetCorrected); % Offset corrected spectra
   hold on
   
end

end



%% Not built-in MATLAB functions (for further details see Sakamoto et al., 2009) 

% Calculation of ESWSample
% ESW(lambda,Tis) in reference
function [ESWSample] = ESW(wavelength, sampleTemperature, calibrateTemperature, ESWCalibrated)

ASWSample = ASW(wavelength, sampleTemperature);
ASWCalibrated = ASW(wavelength, calibrateTemperature);

ESWSample = (ESWCalibrated .* ASWSample) ./ (ASWCalibrated);
end


% Calculation of ASWTemperature 
% ASW(lambda,Tis) and ASW(lambda,Tcal) in reference 
% A,B,C,D are regression parameters (Sakamoto et al, 2009)
function [ASWTemperature] = ASW(wavelength, sampleTemperature)

A = 1.1500276;
B = 0.02840;
C = -0.3101349;
D = 0.001222;

T = sampleTemperature;
W = wavelength-210.0;

ASWTemperature = (((A + B * T) .* exp((C +(D * T)) .* W)));
end


% Calculation of ASESample
% ASE(lambda,Tis) in refrence
function [ASESample] = ASE(ESWSample, sampleSalinity)

ASESample = ESWSample .* sampleSalinity /35;
end









