%% Import data from text file
% Script for importing data from following text files:
%
%   'HE545_absorptionOPUS.txt'
%   'ReferenceNitrate.txt'
%   'ReferenceSalinity.txt'
%

%% Initialize variables.
filename1 = 'HE545_absorptionOPUS.txt'; % insert filename of samples
filename2 = 'ReferenceNitrate.txt'; %insert filename of nitrate reference measurements
filename3 = 'ReferenceSalinity.txt'; %insert filname of salinity reference measurements
delimiter = {' '};

%% Read columns of data as text:
% For more information, see the TEXTSCAN documentation.
formatSpec1 = '%q%q%q%q%q%q%q%q%q%[^\n\r]';
formatSpec = '%s%s%[^\n\r]';

%% Open the text file.
fileID1 = fopen(filename1,'r');
fileID2 = fopen(filename2,'r');
fileID3 = fopen(filename3,'r');

%% Read columns of data according to the format.
% This call is based on the structure of the file used to generate this
% code. If an error occurs for a different file, try regenerating the code
% from the Import Tool.
dataArray1 = textscan(fileID1, formatSpec1, 'Delimiter', delimiter, 'TextType', 'string',  'ReturnOnError', false);
dataArray2 = textscan(fileID2, formatSpec, 'Delimiter', delimiter, 'TextType', 'string',  'ReturnOnError', false);
dataArray3 = textscan(fileID3, formatSpec, 'Delimiter', delimiter, 'TextType', 'string',  'ReturnOnError', false);

%% Close the text file.
fclose(fileID1);
fclose(fileID2);
fclose(fileID3);


%% Absorption spectra
% Convert the contents of columns containing numeric text to numbers.
% Replace non-numeric text with NaN.
raw1 = repmat({''},length(dataArray1{1}),length(dataArray1)-1);
for col1=1:length(dataArray1)-1
    raw1(1:length(dataArray1{col1}),col1) = mat2cell(dataArray1{col1}, ones(length(dataArray1{col1}), 1));
end
numericData1 = NaN(size(dataArray1{1},1),size(dataArray1,2));

for col1=[1,2]
    % Converts text in the input cell array to numbers. Replaced non-numeric
    % text with NaN.
    rawData1 = dataArray1{col1};
    for row1=1:size(rawData1, 1)
        % Create a regular expression to detect and remove non-numeric prefixes and
        % suffixes.
        regexstr1 = '(?<prefix>.*?)(?<numbers>([-]*(\d+[\,]*)+[\.]{0,1}\d*[eEdD]{0,1}[-+]*\d*[i]{0,1})|([-]*(\d+[\,]*)*[\.]{1,1}\d+[eEdD]{0,1}[-+]*\d*[i]{0,1}))(?<suffix>.*)';
        try
            result1 = regexp(rawData1(row1), regexstr1, 'names');
            numbers1 = result1.numbers;
            
            % Detected commas in non-thousand locations.
            invalidThousandsSeparator1 = false;
            if numbers1.contains(',')
                thousandsRegExp1 = '^\d+?(\,\d{3})*\.{0,1}\d*$';
                if isempty(regexp(numbers1, thousandsRegExp1, 'once'))
                    numbers1 = NaN;
                    invalidThousandsSeparator1 = true;
                end
            end
            % Convert numeric text to numbers.
            if ~invalidThousandsSeparator1
                numbers1 = textscan(char(strrep(numbers1, ',', '')), '%f');
                numericData1(row1, col1) = numbers1{1};
                raw1{row1, col1} = numbers1{1};
            end
        catch
            raw1{row1, col1} = rawData1{row1};
        end
    end
end

% Replace non-numeric cells with NaN
R1 = cellfun(@(x) ~isnumeric(x) && ~islogical(x),raw1); % Find non-numeric cells
raw1(R1) = {NaN}; % Replace non-numeric cells


%% Nitrate Reference spectra
% Convert the contents of columns containing numeric text to numbers.
% Replace non-numeric text with NaN.
raw2 = repmat({''},length(dataArray2{1}),length(dataArray2)-1);
for col2=1:length(dataArray2)-1
    raw2(1:length(dataArray2{col2}),col2) = mat2cell(dataArray2{col2}, ones(length(dataArray2{col2}), 1));
end
numericData2 = NaN(size(dataArray2{1},1),size(dataArray2,2));

for col2=[1,2]
    % Converts text in the input cell array to numbers. Replaced non-numeric
    % text with NaN.
    rawData2 = dataArray2{col2};
    for row2=1:size(rawData2, 1)
        % Create a regular expression to detect and remove non-numeric prefixes and
        % suffixes.
        regexstr2 = '(?<prefix>.*?)(?<numbers>([-]*(\d+[\,]*)+[\.]{0,1}\d*[eEdD]{0,1}[-+]*\d*[i]{0,1})|([-]*(\d+[\,]*)*[\.]{1,1}\d+[eEdD]{0,1}[-+]*\d*[i]{0,1}))(?<suffix>.*)';
        try
            result2 = regexp(rawData2(row2), regexstr2, 'names');
            numbers2 = result2.numbers;
            
            % Detected commas in non-thousand locations.
            invalidThousandsSeparator2 = false;
            if numbers2.contains(',')
                thousandsRegExp2 = '^\d+?(\,\d{3})*\.{0,1}\d*$';
                if isempty(regexp(numbers2, thousandsRegExp2, 'once'))
                    numbers2 = NaN;
                    invalidThousandsSeparator2 = true;
                end
            end
            % Convert numeric text to numbers.
            if ~invalidThousandsSeparator2
                numbers2 = textscan(char(strrep(numbers2, ',', '')), '%f');
                numericData2(row2, col2) = numbers2{1};
                raw2{row2, col2} = numbers2{1};
            end
        catch
            raw2{row2, col2} = rawData2{row2};
        end
    end
end

% Replace non-numeric cells with NaN
R2 = cellfun(@(x) ~isnumeric(x) && ~islogical(x),raw2); % Find non-numeric cells
raw2(R2) = {NaN}; % Replace non-numeric cells


%% Salinity Reference spectra
% Convert the contents of columns containing numeric text to numbers.
% Replace non-numeric text with NaN.
raw3 = repmat({''},length(dataArray3{1}),length(dataArray3)-1);
for col3=1:length(dataArray3)-1
    raw3(1:length(dataArray3{col3}),col3) = mat2cell(dataArray3{col3}, ones(length(dataArray3{col3}), 1));
end
numericData3 = NaN(size(dataArray3{1},1),size(dataArray3,2));

for col3=[1,2]
    % Converts text in the input cell array to numbers. Replaced non-numeric
    % text with NaN.
    rawData3 = dataArray3{col3};
    for row3=1:size(rawData3, 1)
        % Create a regular expression to detect and remove non-numeric prefixes and
        % suffixes.
        regexstr3 = '(?<prefix>.*?)(?<numbers>([-]*(\d+[\,]*)+[\.]{0,1}\d*[eEdD]{0,1}[-+]*\d*[i]{0,1})|([-]*(\d+[\,]*)*[\.]{1,1}\d+[eEdD]{0,1}[-+]*\d*[i]{0,1}))(?<suffix>.*)';
        try
            result3 = regexp(rawData3(row3), regexstr3, 'names');
            numbers3 = result3.numbers;
            
            % Detected commas in non-thousand locations.
            invalidThousandsSeparator3 = false;
            if numbers3.contains(',')
                thousandsRegExp3 = '^\d+?(\,\d{3})*\.{0,1}\d*$';
                if isempty(regexp(numbers3, thousandsRegExp3, 'once'))
                    numbers3 = NaN;
                    invalidThousandsSeparator3 = true;
                end
            end
            % Convert numeric text to numbers.
            if ~invalidThousandsSeparator3
                numbers3 = textscan(char(strrep(numbers3, ',', '')), '%f');
                numericData3(row3, col3) = numbers3{1};
                raw3{row3, col3} = numbers3{1};
            end
        catch
            raw3{row3, col3} = rawData3{row3};
        end
    end
end

% Replace non-numeric cells with NaN
R3 = cellfun(@(x) ~isnumeric(x) && ~islogical(x),raw3); % Find non-numeric cells
raw3(R3) = {NaN}; % Replace non-numeric cells


%% Create output variable
AbsorptionRaw = cell2mat(raw1);
ABSORPTINO3 = cell2mat(raw2);
ABSORPTISal = cell2mat(raw3);

%% Clear temporary variables
clearvars ans filename1 filename2 filename3 delimiter formatSpec formatSpec1;
clearvars fileID1 fileID2 fileID3 R1 R2 R3 row1 row2 row3 ;
clearvars dataArray1 dataArray2 dataArray3 rawData1 rawData2 rawData3 ;
clearvars raw1 raw2 raw3 col1 col2 col3 result1 result2 result3 ;
clearvars numericData1 regexstr1 numbers1 invalidThousandsSeparator1 thousandsRegExp1;
clearvars numericData2 regexstr2 numbers2 invalidThousandsSeparator2 thousandsRegExp2;
clearvars numericData3 regexstr3 numbers3 invalidThousandsSeparator3 thousandsRegExp3;



%% Create absorption spectra table
Lambda0=AbsorptionRaw(41,1);
I=find(AbsorptionRaw==Lambda0);

I(:,2)=I(:,1)+199;
k=length(I(:,1));

for i=1:k
absorption(:,i)=AbsorptionRaw((I(i,1):I(i,2)),2);
end


%% Create wavelength variable
W=[I(1,1) I(1,2)];
wavelength=AbsorptionRaw((W(1,1):W(1,2)),1);


%% Create Nitrate reference spectrum
I=find(ABSORPTINO3==Lambda0);
I(:,2)=I(:,1)+199;

k=length(I(:,1));

for i=1:k
NO3ref_all(:,i)=ABSORPTINO3((I(i,1):I(i,2)),2);
end

ReferenceNitrate=mean(NO3ref_all,2);


%% Create Salinity reference spectrum

I=find(ABSORPTISal==Lambda0);
I(:,2)=I(:,1)+199;

k=length(I(:,1));

for i=1:k
Salref_all(:,i)=ABSORPTISal((I(i,1):I(i,2)),2);
end

ReferenceSalinity=mean(Salref_all,2);

%% Clear temporary variables
clearvars I k i W Lambda0 Salref_all NO3ref_all AbsorptionRaw ABSORPTINO3 ABSORPTISal;


%% save variables
save('wavelength.mat','wavelength')
save('absorption.mat','absorption')
save('ReferenceNitrate.mat','ReferenceNitrate')
save('ReferenceSalinity.mat','ReferenceSalinity')






















